#![feature(concat_idents)]

use hercules_rt::runner;
use hercules_rt::HerculesCPURef;
#[cfg(feature = "cuda")]
use hercules_rt::CUDABox;

juno_build::juno!("concat");

fn main() {
    async_std::task::block_on(async {
        let mut r = runner!(concat_entry);
        let mut a_data = [7, 7, 0];
        let mut b_data = [7, 7, 0, 0, 7, 7];
        #[cfg(not(feature = "cuda"))]
        {
            let a = HerculesCPURef::from_slice(&mut a_data);
            let b = HerculesCPURef::from_slice(&mut b_data);
            let output = r.run(3, 6, a, b).await;
            assert_eq!(output, 42);

            const N: usize = 3;
            let arr : Box<[i32]> = (2..=4).collect();
            let arr = HerculesCPURef::from_slice(&arr);

            let mut r = runner!(concat_switch);
            let output = r.run(N as u64, 50, arr.clone()).await;
            let result = output.as_slice::<i32>();
            println!("{:?}", result);
            assert_eq!(result, [0, 1, 2, 3, 4]);

            let output = r.run(N as u64, 30, arr).await;
            let result = output.as_slice::<i32>();
            println!("{:?}", result);
            assert_eq!(result, [2, 3, 4, 0, 1]);
        }
        #[cfg(feature = "cuda")]
        {
            let mut a_data = [7, 7, 0];
            let a = CUDABox::from_cpu_ref(HerculesCPURef::from_slice(&mut a_data));
            let mut b_data = [7, 7, 0, 0, 7, 7];
            let b = CUDABox::from_cpu_ref(HerculesCPURef::from_slice(&mut b_data));
            let output = r.run(3, 6, a.get_ref(), b.get_ref()).await;
            assert_eq!(output, 42);
        }
    });
}

#[test]
fn concat_test() {
    main();
}
